unit Unit4;

interface

uses
  Classes, Forms, Graphics, SysUtils;

type
  TVoxel = Record
            X,Y,Z:Byte;
            Color:Word;
          End;
  TVoxelArray = Array [0..16777215] of TVoxel;
  PVoxelArray = ^TVoxelArray;

  TVoxelObj = class(TObject)
    Anz, AnzPlane:0..16777216;
    Vox:PVoxelArray;
    Plane:PVoxelArray;
    Buf:TBitmap;
    BufAngle:Byte;
    Buffilled:Boolean;
  protected
  procedure Create;
  destructor Destroy;override;
  private
    { Private-Deklarationen }
  public
    procedure CreateTestObj;
    procedure CreatePlane;
    procedure Load(FileName:String);
  end;

implementation

uses Unit1, Unit3;

procedure TVoxelObj.Create;
begin
  Anz:=0;
  Vox:=Nil;
  Buf:=Nil;
  BufAngle:=0;
  BufFilled:=False;
end;

procedure TVoxelObj.CreatePlane;
type
  Voxel2d=Record
            Z:Word;
            Color:Word;
          End;
var
  i, j:0..16777216;
  PlaneMap:Array [0..255, 0..255] of Voxel2d;
  Num:Integer;
begin
  AnzPlane:=0;
  if Anz>0 then begin
    for i:=0 to 255 do begin
      for j:=0 to 255 do begin
        PlaneMap[i, j].Z:=256;
      end;
    end;
    for i:=0 to Anz-1 do begin
      if PlaneMap[Vox^[i].X, Vox^[i].Y].Z>Vox^[i].Z then begin
        PlaneMap[Vox^[i].X, Vox^[i].Y].Z:=Vox^[i].Z;
        PlaneMap[Vox^[i].X, Vox^[i].Y].Color:=Vox^[i].Color;
        Inc(AnzPlane);
      end;
    end;
    GetMem(Plane, AnzPlane*SizeOf(TVoxel));
    Num:=0;
    for i:=0 to Anz-1 do begin
      if PlaneMap[Vox^[i].X, Vox^[i].Y].Z=Vox^[i].Z then begin
        Plane^[Num]:=Vox^[i];
        Inc(Num);
      end;
    end;
  end;
end;

procedure TVoxelObj.Load(FileName:String);
var
  F:File;
  i:0..16777216;
//  pl2:^pixline;
begin
  AssignFile(F,FileName);
  Reset(F,1);
  BlockRead(F,Anz, SizeOf(Anz));
  if Anz>0 then begin
    GetMem(Vox, Anz*SizeOf(TVoxel));
    for i:=0 to Anz-1 do begin
      BlockRead(F, Vox^[i], SizeOf(TVoxel));
    end;
  end;
  Close(f);
  CreatePlane;
  Buf:=TBitmap.Create;
  Buf.Width:=256;
  Buf.Height:=256;
  Buf.PixelFormat:=pf16bit;
{  pl2:=Buf.Scanline[0];
  for i:=0 to Buf.Height-1 do begin
    FillChar(pl2^[-i*Buf.Width], Buf.Width*2, 0);
  end;}
{  Buf.Transparent:=True;
  Buf.TransparentMode:=tmFixed;
  Buf.TransparentColor:=0;
  Buf.ReleaseMaskHandle;}
end;

procedure TVoxelObj.CreateTestObj;
var i,j,k,VWidth,VHeight,VDepth:Integer;
begin
  VWidth:=256;
  VHeight:=256;
  VDepth:=1;
  Anz:=VWidth*VHeight*VDepth;
  GetMem(Vox, Anz*SizeOf(TVoxel));
  for k:=0 to VDepth-1 do begin
    for i:=0 to VWidth-1 do begin
      for j:=0 to VHeight-1 do begin
        Vox^[i*VWidth+j].X:=128-VWidth div 2+i;
        Vox^[i*VWidth+j].Y:=128-VHeight div 2+j;
        Vox^[i*VWidth+j].Z:=128+k;
        Vox^[i*VWidth+j].Color:=(i div 4)*32+j div 8;
      end;
    end;
  end;
end;

destructor TVoxelObj.Destroy;
begin
  if Vox<>Nil then FreeMem(Vox, Anz*SizeOf(TVoxel));
  if Plane<>Nil then FreeMem(Plane, AnzPlane*SizeOf(TVoxel));
  if Buf<>Nil then Buf.Free;
  inherited Destroy;
end;

end.
